

Class clsExcelCell Common

   Instance IsDefaultStyle As Byte
   Instance StyleIndex     As Long
   Instance FontMatched    As Byte
   Instance FontName       As String
   Instance FontSize       As Byte
   Instance FontBold       As Byte
   Instance FontItalic     As Byte
   Instance FontUnderline  As Byte
   Instance FontColor      As String
   Instance CellColor      As String
   Instance Format         As String
   Instance BorderLeft     As Byte
   Instance BorderTop      As Byte
   Instance BorderRight    As Byte
   Instance BorderBottom   As Byte
   Instance BorderColor    As String
   Instance AlignLeft      As Byte
   Instance AlignRight     As Byte
   Instance AlignCenter    As Byte
   Instance Row            As Long
   Instance Col            As Long
   Instance Value          As Variant
   Instance Formula        As String
   Instance MergeAcross    As Byte
   Instance WrapText       As Byte 
   
   
   '//
   '//
   '//
   Class Method Create()
   End Method
   
   '//
   '//
   '//
   Class Method Destroy()
   End Method


   
   '//
   '//
   '//
   Interface iExcelCell: Inherit IUnknown     

      Property Get StyleIndex_Cell() As String
         Property = " ss:StyleID=" & $Dq & "s" & Format$(StyleIndex) & $Dq
      End Property

      PropGetSet(StyleIndex, Long)
      PropGetSet(FontMatched, Byte)     ' flag that the font for this cell matches the default font (omit in xml output)
      PropGetSet(IsDefaultStyle, Byte)
      PropGetSet(FontName, String)
      PropGetSet(FontSize, Byte)
      PropGetSet(FontBold, Byte)
      PropGetSet(FontItalic, Byte)
      PropGetSet(FontUnderline, Byte)
      PropGetSet(FontColor, String)
      PropGetSet(CellColor, String)
      PropGetSet(Format, String)
      PropGetSet(BorderLeft, Byte)
      PropGetSet(BorderTop, Byte)
      PropGetSet(BorderRight, Byte)
      PropGetSet(BorderBottom, Byte)
      PropGetSet(BorderColor, String)
      PropGetSet(AlignLeft, Byte)
      PropGetSet(AlignRight, Byte)
      PropGetSet(AlignCenter, Byte)
      PropGetSet(WrapText, Byte)
      
      PropGetSet(Formula, String)
      PropGetSet(Value, Variant)
      PropGetSet(Row, Long)
      PropGetSet(Col, Long)
      PropGetSet(MergeAcross, Byte)



      Property Get StyleXML() As String
         Local sBorderColor As String
         Local sBorders As String
         Local sAlign   As String
         Local sWrapText As String
         Local sFormat  As String
         Local st       As String
              
         If IsTrue(FontMatched) Then
            ' The font for this cell has matched the default font style for the workbook
            ' therefore there is no need to output those elements. 
            st = _
                 IIf$(IsTrue(FontBold),      " ss:Bold=*1*", "")           & _
                 IIf$(IsTrue(FontItalic),    " ss:Italic=*1*", "")         & _
                 IIf$(IsTrue(FontUnderline), " ss:Underline=*Single*", "") 
         Else      
            st = _
                 IIf$(Len(FontName),         " ss:FontName=*" & FontName & "*", "") & _
                 IIf$(FontSize > 0,          " ss:Size=*" & Format$(FontSize) & "*", "")     & _
                 IIf$(Len(FontColor),        " ss:Color=*" & FontColor & "*", "")  & _
                 IIf$(IsTrue(FontBold),      " ss:Bold=*1*", "")           & _
                 IIf$(IsTrue(FontItalic),    " ss:Italic=*1*", "")         & _
                 IIf$(IsTrue(FontUnderline), " ss:Underline=*Single*", "")
         End If
         If Len(st) Then
            st = "  <Font" & st & "/>" & $CrLf 
         End If   
         
         If Len(CellColor) Then
            st = st & _          
              "  <Interior ss:Color=*" & CellColor & "* ss:Pattern=*Solid*/>" & $CrLf
         End If    
         

         sBorderColor = IIf$( Len(BorderColor), BorderColor, "#000000" )
         sBorders = _
            IIf$(IsTrue(BorderLeft), _
                   "   <Border ss:Position=*Left* ss:LineStyle=*Continuous* ss:Weight=*1* ss:Color=*" & sBorderColor & "*/>" & $CrLf,"") & _
            IIf$(IsTrue(BorderTop), _
                   "   <Border ss:Position=*Top* ss:LineStyle=*Continuous* ss:Weight=*1* ss:Color=*" & sBorderColor & "*/>" & $CrLf,"") & _
            IIf$(IsTrue(BorderRight), _
                   "   <Border ss:Position=*Right* ss:LineStyle=*Continuous* ss:Weight=*1* ss:Color=*" & sBorderColor & "*/>" & $CrLf,"") & _
            IIf$(IsTrue(BorderBottom), _
                   "   <Border ss:Position=*Bottom* ss:LineStyle=*Continuous* ss:Weight=*1* ss:Color=*" & sBorderColor & "*/>" & $CrLf,"")
         If Len(sBorders) Then
            st = st & "  <Borders>" & $CrLf & sBorders & "  </Borders>" & $CrLf          
         End If
            
         sWrapText = IIf$(IsTrue(WrapText), " ss:WrapText=*1*", "")
         If IsTrue(AlignLeft) Then
            sAlign = "  <Alignment ss:Horizontal=*Left* ss:Vertical=*Bottom*" & sWrapText & "/>" & $CrLf
         ElseIf IsTrue(AlignRight) Then   
            sAlign = "  <Alignment ss:Horizontal=*Right* ss:Vertical=*Bottom*" & sWrapText & "/>" & $CrLf
         ElseIf IsTrue(AlignCenter) Then   
            sAlign = "  <Alignment ss:Horizontal=*Center* ss:Vertical=*Bottom*" & sWrapText & "/>" & $CrLf 
         ElseIf IsTrue(WrapText) Then    
            sAlign = "  <Alignment ss:WrapText=*1*/>" & $CrLf 
         End If   
         If Len(sAlign) Then st = st & sAlign
         
         If Len(Format) Then
            sFormat = "  <NumberFormat ss:Format=*" & Format & "*/>" & $CrLf
         End If
         If Len(sFormat) Then st = st & sFormat   

         Replace "*" With $Dq In st

         Property = st
         
      End Property
      


      Property Get ExcelCellXML( ByVal nPrevCol As Long ) As String 
         
         Local sIndex   As String 
         Local sType    As String
         Local sValue   As String
         Local sCell    As String
         Local sData    As String 
         Local sFormula As String
         Local sStyle   As String
         Local sMerge   As String 
         
         ' Determine the type of data in the variant
         Select Case As Long VariantVT(Value)
            Case %VT_I2, %VT_I4, %VT_R4, %VT_R8, %VT_Cy, _
                 %VT_Bool, %VT_DECIMAL, %VT_I1, _
                 %VT_UI1, %VT_UI2, %VT_UI4, %VT_I8, _
                 %VT_UI8, %VT_Int, %VT_UInt
               sType = "Number"
               sValue = Format$(Variant#(Value))
            
            Case Else
                 '%VT_BStr, %VT_Variant, %VT_LPStr, %VT_LPWStr
                 '%VT_Empty, %VT_Null, %VT_Date
               ' If a Format property exists for this cell then it
               ' must be a Date.
               If Len(Format) Then 
                  sValue = Variant$(Value) & "T00:00:00.000"
                  sType = "DateTime"
               Else  
                  sValue = Variant$(Value)
                  sType = "String"        
               End If   
         End Select
         
         sCell = "  <Cell//CELL_INDEX////MERGE_ACROSS////STYLE_ID////CELL_FORMULA//>" & _
                     "<Data ss:Type=*//CELL_TYPE//*>//CELL_TEXT//</Data></Cell>" 
         Replace "*" With $Dq In sCell

   
         Replace "//CELL_TYPE//"   With sType  In sCell
         Replace "//CELL_TEXT//"   With sValue In sCell

         If IsTrue(StyleIndex) Then   ' > 0 therefore not a reference to the Default style
            sStyle = Me.StyleIndex_Cell
         Else
            sStyle = ""
         End If
         Replace "//STYLE_ID//" With sStyle In sCell
         
         ' No need to explicitly set the column index if the current
         ' column is +1 over the previous column. Also don't need
         ' to set the index if this cell is at position 1.
         If (Col = 1) Or (Col = nPrevCol + 1) Then
         Else
            sIndex = " ss:Index=" & $Dq & Format$(Col) & $Dq
         End If
         Replace "//CELL_INDEX//" With sIndex In sCell
         
         If Len(Formula) Then
            sFormula = " ss:Formula=" & $Dq & Formula & $Dq
         End If    
         Replace "//CELL_FORMULA//" With sFormula In sCell 
         
         If MergeAcross > 0 Then
            sMerge = " ss:MergeAcross=" & $Dq & Format$(MergeAcross) & $Dq
         End If    
         Replace "//MERGE_ACROSS//" With sMerge In sCell 
         
         Property = sCell
      End Property

   End Interface
   
End Class
